<?php
/*
Plugin Name: FibroJedi's Sticky Preview and Publish Buttons
Plugin URI: https://wordpress.org/plugins/fibrojedi-sticky-preview-publish-buttons/
Text Domain: fibrojedi-sticky-preview-publish-buttons
Description: Creates a Preview and Publish/Update Button in post creation/post edit, that scrolls down the screen as you scroll. No more scrolling back to the top of the editor to hit preview!
Version: 1.0.2
Stable tag: 1.0.2
Tested up to: 6.1
Tags: post-editing, classic-editor, editor, admin-buttons, publish-button, back-office, ui
Author: FibroJedi
Author URI: https://fibrojedi.me.uk/
Donate Link: https://ko-fi.com/fibrojedi
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/

function fibrojedi_sppb_load_css(){
  wp_register_style( 'fibrojedi-sppb-css', plugin_dir_url( __FILE__ ) . 'fibrojedi-sppb.css','','', 'screen' );
  wp_enqueue_style( 'fibrojedi-sppb-css' );
  // Attempt to use CSS to set the 'top' attribute rather than JavaScript. Tricky as add_meta_box doesn't allow for extra html attributes.
  $fibrojedi_sppb_position = get_option('fibrojedi_sppb_position'); // how far from the top it's positioned
  $fibrojedi_sppb_opacity  = get_option('fibrojedi_sppb_opacity');
  $fibrojedi_sppb_slider   = get_option('fibrojedi_sppb_slider'); // whether the sliding effect is on or off.
  if($fibrojedi_sppb_slider===true || $fibrojedi_sppb_slider=="true"){
   // no need to add sliding CSS as that's the default behaviour
   $fibrojedi_css_from_settings = "#fibrojedi_sppb{ top:". $fibrojedi_sppb_position . "%;opacity:".$fibrojedi_sppb_opacity.";}";
  }
  else{
    // overwrite the @media definition (!important)
    $fibrojedi_css_from_settings = "#fibrojedi_sppb{ top:". $fibrojedi_sppb_position . "%;opacity:".$fibrojedi_sppb_opacity.";right:0 !important;}";
  }
  wp_add_inline_style('fibrojedi-sppb-css',$fibrojedi_css_from_settings);
}

add_action( 'admin_enqueue_scripts', 'fibrojedi_sppb_load_css' );



// Create the Meta Box
function fibrojedi_sppb_add() {
    /* Replace ALL except 'post' with your own, until I get options sorted */
    $screens = [ 'post','lotro_cosmetic','lotro_mount','enemy' ];
    //$fibrojedi_sppb_position = get_option('fibrojedi_sppb_position') . "%";
    foreach ( $screens as $screen ) {
        add_meta_box(
            'fibrojedi_sppb',                 // Unique ID
            'Sticky Buttons',      // Box title
            'fibrojedi_sppb_html',  // Content callback, must be of type callable
            $screen,
            'side',
            'high'                            // priority
        );
    }
}
add_action( 'add_meta_boxes', 'fibrojedi_sppb_add' );

function fibrojedi_word_count_last_save($post_id_received){
  //global $post;
  /*
   For 1.0 I should be able to get this from the last revision, then get it via AJAX
   on condition it does not reduce performance, anyway.
   wp_get_post_revisions();

   But first, I needed to change how I got the word count from WP 5.9
  */
  //if(is_object($post)){
$word_count_last_save = 0;
$post_content_grabbed = '';
$post_content_other = '';
$post_content_other_count = 0;
global $wp_query;
$wp_query = new WP_Query();
  $post_content_grabbed = get_the_content(null,true,$post_id_received);
  //$post_content_grabbed = $post_data->post_content;
  $post_content_other = strip_tags(do_shortcode($post_content_grabbed));
  //$post_content = get_the_content($post_id);
  //in-built strip_tags
  /*
   Use the array below to remove any shortcodes you do not want expanding.
   I've replaced them with spaces to prevent actual words joining together.
   Or try to, at least.
  */
  $shortcodes_to_remove = array(
    '[fj_dfi]',
    '[menu_anchor]',
    '[fj_toc]',
    '[uplink]'
  );
  foreach($shortcodes_to_remove as $shortcode_to_remove){
    $post_content_grabbed = str_replace($shortcode_to_remove,' ',$post_content_grabbed);
  }
  //$post_content = strip_tags($post_data->post_content);
  $post_content_grabbed = do_shortcode($post_content_grabbed);
  $post_content_grabbed = strip_tags($post_content_grabbed);

  //v0.5 try to remove any remnant unrecognised shortcodes
  $post_content_grabbed = preg_replace("/\[[^\]]+\]/","",$post_content_grabbed);

  // remove dashes that are surrounded by spaces
  $post_content_grabbed = str_replace(" - "," ",$post_content_grabbed);

  // quirk: my previous method saw slashed words, such as "and/or" as 3 words, not 2.
  $post_content_grabbed = str_replace("/"," ",$post_content_grabbed);

  // quirk: TL;DR is counted as ONE word by the editor, but 2 words by my calculator.
  $post_content_grabbed = str_replace("TL;DR","TLDR",$post_content_grabbed);

  // count words after last modification
  $word_count_last_save = str_word_count($post_content_grabbed);
  $post_content_other_count =str_word_count($post_content_other);
  return array($word_count_last_save,$post_content_grabbed,$post_content_other,$post_content_other_count);
 //} //$post exists
 //else{
//   return array(0,0,0,0);
 //}
}

function fibrojedi_sppb_html( $post ) {
    /*
     Time to get the new options and implement them.
    */
    // 1. Show Word Count
    $show_wordcount = get_option('fibrojedi_sppb_wordcount_show');
    $html_wordcount = '';
    if($show_wordcount===true || $show_wordcount=="true"){
      if(isset($_GET['post'])){
        $post_id_to_send = $_GET['post'];
        $word_count = fibrojedi_word_count_last_save($post_id_to_send);
      }
      else{
        $word_count = array(0,0,0,0); // don't count new post
      }
      $html_wordcount = '<small><span class="word-count" title="'.esc_html__('Approximate','fibrojedi-sticky-preview-publish-buttons').'!"><strong>'.esc_html__('Words','fibrojedi-sticky-preview-publish-buttons').':</strong> '.number_format($word_count[0],0,".",",") . '</span> </small>';
    }
    // 2. Show/Hide Settings Link per...wait for it...Settings.
    $fibrojedi_sppb_show_settings = get_option('fibrojedi_sppb_settings_show');
    $fibrojedi_sppb_settings_link = '';
    if($fibrojedi_sppb_show_settings===true || $fibrojedi_sppb_show_settings=="true"){
      $fibrojedi_sppb_settings_link = '<a href="'.admin_url('options-general.php?page=fibrojedi-sppb').'" target="_blank" title="Opens in a new tab/window so your editing session is not interrupted.">Settings</a>';
    }
    // 3. Change Distance from the Top: we used standard Wordpress PHP and CSS so we're going to have to dynamically shift it.
    $fibrojedi_sppb_position = get_option('fibrojedi_sppb_position');
    $sppb_position = $fibrojedi_sppb_position . "%"; // continues at the bottom of this function.

    // 4. New Link to Custom Fields (Make an option later)
    $other_shortcuts = "";
    $fibrojedi_sppb_show_cf_shortcut = "true";
    if($fibrojedi_sppb_show_cf_shortcut == "true"){
      $other_shortcuts .= '<a href="#postcustom" title="Skip to Custom Fields">Custom Fields</a>';
    }

    // 5. Compile other shortcuts
    $all_extra_links = $html_wordcount.$fibrojedi_sppb_settings_link.$other_shortcuts;
    $all_extra_links = str_replace('<a','| <a',$all_extra_links);
    $all_extra_links = str_replace('</a',' </a',$all_extra_links);
    ?>
    <p><?php echo '<small>'.$all_extra_links.'</small>'; ?><!--[<a href="#" onclick="autosave();event.preventDefault();" title="Try to trigger an AutoSave: in-development">S</a>] --><a href="javascript:document.getElementById('fibrojedi_sppb').style.display='none';event.preventdefault();" title="Hides this box until the next save/publish action." class="fjsppb-close">X</a></small></p>
    <div id="fibrojedi-sppb"><a class="button" href="#" onclick="if(document.getElementById('title').value==''){alert('Cannot preview until title/permalink are added');}else{document.getElementById('post-preview').click();event.preventDefault();}">Preview Changes<span class="screen-reader-text"> (opens in a new tab)</span></a><?php

    if(get_post_status($post->ID)=="draft" || get_post_status($post->ID)=="auto-draft"){ ?>
      <a class="button" href="#" onclick="document.getElementById('save-post').click();event.preventDefault();">Save Draft</a><?php
    } ?><hr><?php

    /*
    Integrate publish confirm option.
    */
    $fibrojedi_publish_confirm = get_option('fibrojedi_publish_confirm'); // true or false (boolean)
    if(get_post_status($post->ID)=="publish"){
      if($fibrojedi_publish_confirm===true || $fibrojedi_publish_confirm=="true"){
      ?><a class="button" href="#" onclick="if(window.confirm('Are you sure you want to Update?')){document.getElementById('publish').click();}event.preventDefault();" type="submit">Update</a><?php
     }
     else{
       ?><a class="button" href="#" onclick="document.getElementById('publish').click();event.preventDefault();" type="submit">Update</a><?php
     }
    }
    else{
      if($fibrojedi_publish_confirm===true || $fibrojedi_publish_confirm=="true"){ ?>
      <a class="button button-primary button-large" href="#" onclick="if(window.confirm('Are you sure you want to Publish?')){document.getElementById('publish').click();}event.preventDefault();" type="submit">Publish</a>
    <?php }
    else{
      ?><a class="button button-primary button-large" href="#" onclick="document.getElementById('publish').click();event.preventDefault();" type="submit">Publish</a><?php
     }
   }
   //echo $other_shortcuts;
    ?><script>document.getElementById('fibrojedi_sppb').class="postbox fibrojedi_sppb_metabox";</script></div><?php
}

// ADD LINKS UNDER PLUGIN NAME
function fibrojedi_sppb_add_plugin_link($current_links){
    $new_actions = array(
      '<a href="' . admin_url( 'options-general.php?page=fibrojedi-sppb' ) . '">Settings</a>',
    );
    return array_merge($current_links, $new_actions);
}

// ADD LINKS UNDER PLUGIN DESCRIPTION
function fibrojedi_sppb_set_plugin_meta($links, $file)
{
  static $plugin;
  $plugin = plugin_basename(__FILE__);
  if ($file == $plugin) {
    $links[] = '<a href="https://ko-fi.com/FibroJedi">Tip a Coffee</a>';
  }
  return $links;
}

function fibrojedi_sppb_register_settings(){
  add_option( 'fibrojedi_sppb_wordcount_show', false);
  add_option( 'fibrojedi_sppb_position', "80");
  add_option( 'fibrojedi_sppb_settings_show', true);
  add_option( 'fibrojedi_sppb_opacity', 1);
  add_option( 'fibrojedi_sppb_slider', true);
  add_option( 'fibrojedi_publish_confirm', true);


  //register_setting( string $option_group, string $option_name, array $args = array() )
  register_setting('fibrojedi_sppb_options', 'fibrojedi_sppb_wordcount_show', $args = array('type'=>'boolean','default'=>false));
  register_setting('fibrojedi_sppb_options', 'fibrojedi_sppb_settings_show', $args = array('type'=>'boolean','default'=>true));
  register_setting('fibrojedi_sppb_options', 'fibrojedi_sppb_position', $args = array('type'=>'string','default'=>'80'));
  register_setting('fibrojedi_sppb_options', 'fibrojedi_sppb_opacity', $args = array('default'=>'1'));
  register_setting('fibrojedi_sppb_options', 'fibrojedi_sppb_slider', $args = array('type'=>'boolean','default'=>true));
  register_setting('fibrojedi_sppb_options', 'fibrojedi_publish_confirm', $args = array('type'=>'boolean','default'=>true));
}
add_action('admin_init','fibrojedi_sppb_register_settings');

function fibrojedi_sppb_settings_page(){
  ?>
  <div class="fibrojedi_sppb_options"><h1><span class="dashicons dashicons-image-flip-vertical"></span>  FibroJedi's Sticky Preview &amp; Publish Buttons</h1>
  <strong><em>Version: 1.0.2</em></strong>
  <h2>Personalise Your Tab</h2>
  <p>I finally got around to adding in options where I had made features I needed, but potentially not everyone did. So, here goes!</p>
  <form method="post" action="options.php">
  <?php settings_fields( 'fibrojedi_sppb_options' ); ?>
  <table border=0>
    <tr valign="top">
      <td scope="row">
        <label for="fibrojedi_sppb_wordcount_show">Show Word Count? </label>
      </td>
      <td><?php
         $show_wordcount = get_option('fibrojedi_sppb_wordcount_show');
        ?><select id="fibrojedi_sppb_wordcount_show" name="fibrojedi_sppb_wordcount_show">
          <option value="true"<?php if($show_wordcount==true || $show_wordcount=="true"){ echo ' selected="selected"'; } ?>>Yes</option>
          <option value="false"<?php if($show_wordcount==false || $show_wordcount=="false"){ echo ' selected="selected"'; } ?>>No</option>
        </select>
      </td>
    </tr>
      <tr valign="top">
        <td scope="row">
          <label for="fibrojedi_sppb_position">Distance from the Top: </label>
        </td>
        <td><?php
           $fibrojedi_sppb_position = get_option('fibrojedi_sppb_position');
          ?><select id="fibrojedi_sppb_position" name="fibrojedi_sppb_position">
            <option value="10"<?php if($fibrojedi_sppb_position=="10" || $fibrojedi_sppb_position==10){ echo ' selected'; } ?>>10%</option>
            <option value="20"<?php if($fibrojedi_sppb_position=="20" || $fibrojedi_sppb_position==20){ echo ' selected'; } ?>>20%</option>
            <option value="30"<?php if($fibrojedi_sppb_position=="30" || $fibrojedi_sppb_position==30){ echo ' selected'; } ?>>30%</option>
            <option value="40"<?php if($fibrojedi_sppb_position=="40" || $fibrojedi_sppb_position==40){ echo ' selected'; } ?>>40%</option>
            <option value="50"<?php if($fibrojedi_sppb_position=="50" || $fibrojedi_sppb_position==50){ echo ' selected'; } ?>>50%</option>
            <option value="60"<?php if($fibrojedi_sppb_position=="60" || $fibrojedi_sppb_position==60){ echo ' selected'; } ?>>60%</option>
            <option value="70"<?php if($fibrojedi_sppb_position=="70" || $fibrojedi_sppb_position==70){ echo ' selected'; } ?>>70%</option>
            <option value="80"<?php if($fibrojedi_sppb_position=="80" || $fibrojedi_sppb_position==80){ echo ' selected'; } ?>>80%</option>
          </select>
        </td>
      </tr>
      <tr valign="top">
        <td scope="row">
          <label for="fibrojedi_sppb_slider">Enable Sliding Effect<br><small>(only applies to screens/resolutions >=1200px wide)</small> </label>
        </td>
        <td><?php
           $fibrojedi_sppb_slider = get_option('fibrojedi_sppb_slider');
          ?><select id="fibrojedi_sppb_slider" name="fibrojedi_sppb_slider">
            <option value="true"<?php if($fibrojedi_sppb_slider==true || $fibrojedi_sppb_slider=="true"){ echo ' selected="selected"'; } ?>>Yes</option>
            <option value="false"<?php if($fibrojedi_sppb_slider==false || $fibrojedi_sppb_slider=="false"){ echo ' selected="selected"'; } ?>>No</option>
          </select>
        </td>
      </tr>
      <tr valign="top">
        <td scope="row">
          <label for="fibrojedi_sppb_opacity">Opacity * </label>
        </td>
        <td><?php
           $fibrojedi_sppb_position = get_option('fibrojedi_sppb_opacity');
          ?><select id="fibrojedi_sppb_opacity" name="fibrojedi_sppb_opacity">
            <option value="0.1"<?php if($fibrojedi_sppb_position=="0.1" || $fibrojedi_sppb_position==0.1){ echo ' selected'; } ?>>10%</option>
            <option value="0.2"<?php if($fibrojedi_sppb_position=="0.2" || $fibrojedi_sppb_position==0.2){ echo ' selected'; } ?>>20%</option>
            <option value="0.3"<?php if($fibrojedi_sppb_position=="0.3" || $fibrojedi_sppb_position==0.3){ echo ' selected'; } ?>>30%</option>
            <option value="0.4"<?php if($fibrojedi_sppb_position=="0.4" || $fibrojedi_sppb_position==0.4){ echo ' selected'; } ?>>40%</option>
            <option value="0.5"<?php if($fibrojedi_sppb_position=="0.5" || $fibrojedi_sppb_position==0.5){ echo ' selected'; } ?>>50%</option>
            <option value="0.6"<?php if($fibrojedi_sppb_position=="0.6" || $fibrojedi_sppb_position==0.6){ echo ' selected'; } ?>>60%</option>
            <option value="0.7"<?php if($fibrojedi_sppb_position=="0.7" || $fibrojedi_sppb_position==0.7){ echo ' selected'; } ?>>70%</option>
            <option value="0.8"<?php if($fibrojedi_sppb_position=="0.8" || $fibrojedi_sppb_position==0.8){ echo ' selected'; } ?>>80%</option>
            <option value="0.9"<?php if($fibrojedi_sppb_position=="0.9" || $fibrojedi_sppb_position==0.9){ echo ' selected'; } ?>>90%</option>
            <option value="1"<?php if($fibrojedi_sppb_position=="1" || $fibrojedi_sppb_position==1){ echo ' selected'; } ?>>100%</option>
          </select>
        </td>
      </tr>
      <tr valign="top">
        <td scope="row">
          <label for="fibrojedi_sppb_settings_show">Show Settings Link? </label>
        </td>
        <td><?php
           $show_settings = get_option('fibrojedi_sppb_settings_show');
          ?><select id="fibrojedi_sppb_settings_show" name="fibrojedi_sppb_settings_show" fj-data="<?php echo $show_settings; ?>">
            <option value="true"<?php if($show_settings===true || $show_settings=="true"){ echo ' selected="selected"'; } ?>>Yes</option>
            <option value="false"<?php if($show_settings===false || $show_settings=="false"){ echo ' selected="selected"'; } ?>>No</option>
          </select>
        </td>
      </tr>
      <tr valign="top">
        <td scope="row">
          <label for="fibrojedi_publish_confirm">Show confirmation dialog when pressing <strong>Publish</strong> or <strong>Update</strong>?</label>
        </td>
        <td><?php
           $fibrojedi_publish_confirm = get_option('fibrojedi_publish_confirm');
          ?><select id="fibrojedi_publish_confirm" name="fibrojedi_publish_confirm">
            <option value="true"<?php if($fibrojedi_publish_confirm==true || $fibrojedi_publish_confirm=="true"){ echo ' selected="selected"'; } ?>>Yes</option>
            <option value="false"<?php if($fibrojedi_publish_confirm==false || $fibrojedi_publish_confirm=="false"){ echo ' selected="selected"'; } ?>>No</option>
          </select>
        </td>
      </tr>
  </table>
  <?php
        submit_button();
  ?>
</form>
<p>* <strong>Opacity:</strong> this is how see-through the tab is when you cursor is <strong>not</strong> hovering over it.</p>
<p>If you primarily use a tablet or other device without a cursor, set this high, or even 100%.</p>
<p><strong>Opacity will always be 100% when your cursor is over it</strong>.</p></div>

<?php
}

function fibrojedi_sppb_settings_menu(){
  //add_submenu_page( string $parent_slug, string $page_title, string $menu_title, string $capability, string $menu_slug, callable $function = '', int $position = null )
  // add settings to the settings tab rather than yet another top-level one.
  add_submenu_page('options-general.php','FibroJedi\'s Sticky Preview &amp; Publish Buttons', 'Sticky Preview Buttons', 'manage_options', 'fibrojedi-sppb', 'fibrojedi_sppb_settings_page',null);
}
// no point loading these on front-end, so don't!
if(is_admin()){
  add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'fibrojedi_sppb_add_plugin_link', 10, 2);
  add_filter('plugin_row_meta', 'fibrojedi_sppb_set_plugin_meta', 10, 2);
  add_action('admin_menu', 'fibrojedi_sppb_settings_menu');
}
?>

